<?php

class User_Model extends AbstractModel {

	private $logged_in;

	private $id;
	private $username;
	private $password;
	private $email;
	private $zipCode;

	function __construct()
	{
		parent::__construct();
		if (!session_id())
			session_start();

		$this->initializeLogin();
		$this->initializeFromDb();
	}

	// populate data with user login information from sessions and/or cookies.
	private function initializeLogin()
	{
		if (isset($_SESSION['user_id'])) {
			$this->logged_in = true;
			$this->id = intval($_SESSION['user_id']);
		} else {
			$this->logged_in = false;
		}
	}

	// using the user's login information, populate user model with data.
	private function initializeFromDb()
	{
		if ($this->isLoggedIn()) {
			$userdata = $this->db->queryRow(
				'select id, username, password, email, zipCode
				from user
				where id = ?',
				array($this->id));
			if (!$userdata) {
				// todo: error handling on invalid login.
				// todo: check password against database (to prevent user changing the username in their cookie).
				trigger_error('Error initializing user information from database.');
				return;
			}

			$this->username = $userdata['username'];
			$this->password = $userdata['password'];
			$this->email = $userdata['email'];
			$this->zipCode = $userdata['zipCode'];
		} else {
			$this->id = -1;
			$this->username = 'Anonymous';
			$this->password = '';
			$this->email = '';
			if (isset($_SESSION['zipCode']))
				$this->zipCode = ($_SESSION['zipCode']);
			else if (isset($_REQUEST['zipCode']))
				$this->zipCode = ($_REQUEST['zipCode']);
			else
				$this->zipCode = '';
		}
	}

	function login($username, $password)
	{
		Session_Handler::start();

		// check that the login credentials are valid.
		$result = $this->db->queryRow(
			'select id, password
			from user
			where username = ?', array($username));
		if (!$result || $result['password'] != $password){
			Session_Handler::addError('Invalid username or password.', 'login');
			return false;
		}

		$this->id = $result['id'];

		// login using session.
		$_SESSION['user_id'] = $this->id;
		$this->logged_in = true;

		return true;
	}

	function logout()
	{
		Session_Handler::end();
		$this->logged_in = false;
	}

	function register($username, $password, $email = null)
	{
		// check that username is valid
		$this->db->query(
			'insert into
				user (username, password)
				values (?, ?)',
				array($username, md5($password)));
		if (!$this->db->lastInsertId()) {
			Session_Handler::addError('Error creating account. That username already taken.', 'login');
			return false;
		}
		
		return true;
	}

	function isLoggedIn()
	{
		return $this->logged_in;
	}

	// getters and setters
	function id()
	{
		if (!$this->isLoggedIn()) {
			// todo: error if accessing id without being logged in
			trigger_error('Accessing ID without being logged in.');
		}

		return $this->id;
	}

	function username()
	{
		return $this->username;
	}

	function zipCode($set = null) {
		if ($set) {
			$this->zipCode = $set;

			if ($this->isLoggedIn() && is_int($this->zipCode)) {
				$this->db->query('update user set zipCode = ? where id = ?', array($set, $this->id()));
			}
		}

		// todo: zipCode should be retrievable from request vars/cookie if user not logged in.
		if ($this->zipCode) {
			return $this->zipCode;
		} else {
			if (Url::get('zipCode')) {
				return Url::get('zipCode');
			}
		}

		// return null as default
		return null;
	}

	function email($set = null) {
		if ($set) {
			$this->email = $set;
			if ($this->isLoggedIn()) {
				$this->db->query('update user set email = ? where id = ?', array($set, $this->id()));
			}

		}

		return $this->email;
	}

	function password($newPassword) {
		//
		// validate password
		// (7 <= length <= 20)
		// contains combination of [a-zA-Z0-9!@#$%^&*()-_+=]
		$safe = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-_+=";

		//
		// update password in database.
		if (strlen($newPassword) < 21 && strlen($newPassword) > 6 && str_replace($safe, "", $newPassword) == "") {
			if ($this->isLoggedIn()) {
				$this->db->query('update user set password = ? where id = ?', array(md5($newPassword), $this->id()));
			}
		}

		// return true if successful, false otherwise.
		return true;
	}

	function getArray()
	{
		return array(
			'id' => $this->id,
			'username' => $this->username,
			'password' => $this->password,
			'email' => $this->email,
			'zipCode' => $this->zipCode,
			'isLoggedIn' => $this->isLoggedIn(),
		);
	}
}

?>