<?php

/* Session Handler abstracts the process of setting and getting cookie and session vars. */
/* Session Handler is also used to "leave messages" for the next page shown, like error messages. */
class Session_Handler {

	/* Leave a message to be shown by the next page view */
	static function addMessage($message)
	{
		self::start();

		if (!isset($_SESSION['messages'])) {
			$_SESSION['messages'] = array();
		}
		$_SESSION['messages'][] = array(
			'message' => $message,
		);
	}

	/* Leave an error message, along with a "type" to identify what kind of error it is (i.e. "login") */
	static function addError($error, $type)
	{
		self::start();

		if (!isset($_SESSION['errors'])) {
			$_SESSION['errors'] = array();
		}
		$_SESSION['errors'][] = array(
			'message' => $error,
			'type' => $type,
		);
	}

	static $_messages;
	static $_errors;

	/* Retrieve the messages left in the previous page view */
	static function getMessages()
	{
		self::start();
		if (self::$_messages)
			return self::$_messages;

		if (isset($_SESSION['messages'])) {
			self::$_messages = $_SESSION['messages'];
			unset($_SESSION['messages']);
			return self::$_messages;
		}

		return array(); // default
	}

	static function getErrors()
	{
		self::start();
		if (self::$_errors)
			return self::$_errors;

		if (isset($_SESSION['errors'])) {
			self::$_errors = $_SESSION['errors'];
			unset($_SESSION['errors']);
			return self::$_errors;
		}

		return array(); // default
	}

	static function setCookie($varname, $value, $expire = 0)
	{
		if (!$varname)
			return false;

		return setcookie($varname, $value, $expire);
	}

	static function getCookie($varname)
	{
		if (isset($_COOKIE[$varname])) {
			return $_COOKIE[$varname];
		} else {
			return null;
		}
	}

	static function setSession($varname, $value)
	{
		if ($varname && self::start()) {
			$_SESSION[$varname] = $value;
			return true;
		} else {
			return false;
		}
	}

	static function getSession($varname)
	{
		if ($varname && self::start()) {
			if (isset($_SESSION[$varname])) {
				return $_SESSION[$varname];
			} else {
				return null;
			}
		}
	}

	static function start() {
		if (!session_id()) {
			return session_start();
		} else {
			return true;
		}
	}
	
	static function end()
	{
		if (session_id())
			return session_destroy();
		else
			return true;
	}
}

?>